@@ -59,7 +59,7 @@ class AlgoliaIndex(object):
     index_name = None
 
     # Use to specify the settings of the index.
-    settings = {}
+    settings = None
 
     # Used to specify if the instance should be indexed.
     # The attribute should be either:
@@ -80,10 +80,13 @@ class AlgoliaIndex(object):
         self.__named_fields = {}
         self.__translate_fields = {}
 
+        if self.settings is None:  # Only set settings if the actual index class does not define some
+            self.settings = {}
+
         try:
-            all_model_fields = model._meta.get_all_field_names()
-        except AttributeError:  # get_all_field_names is removed in Django >= 1.10
-            all_model_fields = [f.name for f in model._meta.get_fields()]
+            all_model_fields = [f.name for f in model._meta.get_fields() if not f.is_relation]
+        except AttributeError:  # get_fields requires Django >= 1.8
+            all_model_fields = [f.name for f in model._meta.local_fields]
 
         if isinstance(self.fields, str):
             self.fields = (self.fields,)
@@ -125,7 +128,7 @@ class AlgoliaIndex(object):
                                                             self.fields)))
 
         # Check custom_objectID
-        if self.custom_objectID in chain(['pk'], all_model_fields):
+        if self.custom_objectID in chain(['pk'], all_model_fields) or hasattr(model, self.custom_objectID):
             self.objectID = get_model_attr(self.custom_objectID)
         else:
             raise AlgoliaIndexError('{} is not a model field of {}'.format(
@@ -217,10 +220,9 @@ class AlgoliaIndex(object):
 
             if self.tags:
                 if callable(self.tags):
-                    self.tags = self.tags(instance)
-                if not isinstance(self.tags, list):
-                    self.tags = list(self.tags)
-                tmp['_tags'] = self.tags
+                    tmp['_tags'] = self.tags(instance)
+                if not isinstance(tmp['_tags'], list):
+                    tmp['_tags'] = list(tmp['_tags'])
 
         logger.debug('BUILD %s FROM %s', tmp['objectID'], self.model)
         return tmp
@@ -278,23 +280,23 @@ class AlgoliaIndex(object):
         For more information about partial_update_object:
         https://github.com/algolia/algoliasearch-client-python#update-an-existing-object-in-the-index
         """
-        if self.should_index:
-            if not self.should_index(instance):
-                # Should not index, but since we don't now the state of the
-                # instance, we need to send a DELETE request to ensure that if
-                # the instance was previously indexed, it will be removed.
-                self.delete_record(instance)
-                return
+        if not self._should_index(instance):
+            # Should not index, but since we don't now the state of the
+            # instance, we need to send a DELETE request to ensure that if
+            # the instance was previously indexed, it will be removed.
+            self.delete_record(instance)
+            return
 
         try:
             if update_fields:
                 obj = self.get_raw_record(instance,
                                           update_fields=update_fields)
-                self.__index.partial_update_object(obj)
+                result = self.__index.partial_update_object(obj)
             else:
                 obj = self.get_raw_record(instance)
-                self.__index.save_object(obj)
+                result = self.__index.save_object(obj)
             logger.info('SAVE %s FROM %s', obj['objectID'], self.model)
+            return result
         except AlgoliaException as e:
             if DEBUG:
                 raise e
@@ -359,7 +361,19 @@ class AlgoliaIndex(object):
             if DEBUG:
                 raise e
             else:
-                logger.warning('ERROR DURING SEARCH: %s', e)
+                logger.warning('ERROR DURING SEARCH ON %s: %s', self.index_name, e)
+
+    def get_settings(self):
+        """Returns the settings of the index."""
+        try:
+            logger.info('GET SETTINGS ON %s', self.index_name)
+            return self.__index.get_settings()
+        except AlgoliaException as e:
+            if DEBUG:
+                raise e
+            else:
+                logger.warning('ERROR DURING GET_SETTINGS ON %s: %s',
+                               self.model, e)
 
     def set_settings(self):
         """Applies the settings to the index."""
@@ -373,7 +387,7 @@ class AlgoliaIndex(object):
             if DEBUG:
                 raise e
             else:
-                logger.warning('SETTINGS NOT APPLYED ON %s: %s',
+                logger.warning('SETTINGS NOT APPLIED ON %s: %s',
                                self.model, e)
 
     def clear_index(self):
@@ -387,6 +401,16 @@ class AlgoliaIndex(object):
             else:
                 logger.warning('%s NOT CLEARED: %s', self.model, e)
 
+    def wait_task(self, task_id):
+        try:
+            self.__index.wait_task(task_id)
+            logger.info('WAIT TASK %s', self.index_name)
+        except AlgoliaException as e:
+            if DEBUG:
+                raise e
+            else:
+                logger.warning('%s NOT WAIT: %s', self.model, e)
+
     def reindex_all(self, batch_size=1000):
         """
         Reindex all the records.
@@ -395,6 +419,19 @@ class AlgoliaIndex(object):
         a method `get_queryset` in your subclass. This can be used to optimize
         the performance (for example with select_related or prefetch_related).
         """
+        should_keep_synonyms = False
+        should_keep_rules = False
+        try:
+            if not self.settings:
+                self.settings = self.get_settings()
+                logger.debug('Got settings for index %s: %s', self.index_name, self.settings)
+            else:
+                logger.debug("index %s already has settings: %s", self.index_name, self.settings)
+        except AlgoliaException as e:
+            if any("Index does not exist" in arg for arg in e.args):
+                pass  # Expected, let's clear and recreate from scratch
+            else:
+                raise e  # Unexpected error while getting settings
         try:
             if self.settings:
                 replicas = self.settings.get('replicas', None)
@@ -412,6 +449,19 @@ class AlgoliaIndex(object):
 
                 self.__tmp_index.wait_task(self.__tmp_index.set_settings(self.settings)['taskID'])
                 logger.debug('APPLY SETTINGS ON %s_tmp', self.index_name)
+            rules = []
+            synonyms = []
+            for r in self.__index.iter_rules():
+                rules.append(r)
+            for s in self.__index.iter_synonyms():
+                synonyms.append(s)
+            if len(rules):
+                logger.debug('Got rules for index %s: %s', self.index_name, rules)
+                should_keep_rules = True
+            if len(synonyms):
+                logger.debug('Got synonyms for index %s: %s', self.index_name, rules)
+                should_keep_synonyms = True
+
             self.__tmp_index.clear_index()
             logger.debug('CLEAR INDEX %s_tmp', self.index_name)
 
@@ -424,9 +474,8 @@ class AlgoliaIndex(object):
                 qs = self.model.objects.all()
 
             for instance in qs:
-                if self.should_index:
-                    if not self.should_index(instance):
-                        continue  # should not index
+                if not self._should_index(instance):
+                    continue  # should not index
 
                 batch.append(self.get_raw_record(instance))
                 if len(batch) >= batch_size:
@@ -454,6 +503,14 @@ class AlgoliaIndex(object):
                     logger.debug("RESTORE SLAVES")
                 if should_keep_replicas or should_keep_slaves:
                     self.__index.set_settings(self.settings)
+                if should_keep_rules:
+                    response = self.__index.batch_rules(rules, forward_to_replicas=True)
+                    self.__index.wait_task(response['taskID'])
+                    logger.info("Saved rules for index %s with response: {}".format(response), self.index_name)
+                if should_keep_synonyms:
+                    response = self.__index.batch_synonyms(synonyms, forward_to_replicas=True)
+                    self.__index.wait_task(response['taskID'])
+                    logger.info("Saved synonyms for index %s with response: {}".format(response), self.index_name)
             return counts
         except AlgoliaException as e:
             if DEBUG:
